package request

import (
	"context"
	"crypto/tls"
	"fmt"
	"log"

	"encoding/json"
	"io/ioutil"

	driver "github.com/arangodb/go-driver"
	"github.com/arangodb/go-driver/http"
)

//attr interface{}

//map[1 , 2 , 3 map [ .. ]]

/*
SendAQLQuery send AQL string query to database and returns a JSON object in a general format

Parameters: AQLQuery is a string containing the query that will be send to the database

Return: a map with two entries: "nodes" with a list of vertices/nodes and "edges" with a list of edges
that will be returned to the frontend
*/
func (s *Service) SendAQLQuery(AQLQuery string) (*map[string][]Document, error) {
	var queryResult = make(map[string][]Document)
	conn, err := http.NewConnection(http.ConnectionConfig{
		Endpoints: []string{"https://aae8f5c054da.arangodb.cloud:8529"},
		TLSConfig: &tls.Config{InsecureSkipVerify: true},
	})
	if err != nil {
		log.Println("could not connect to database") // Handle error
		return nil, err
	}
	c, err := driver.NewClient(driver.ClientConfig{
		Connection:     conn,
		Authentication: driver.BasicAuthentication("root", "kXR0PFepIVm3EqgQ2MVc"),
	})
	if err != nil {
		log.Println("Could not log in to the ArangoDB") // Handle error
		return nil, err
	}

	ctx := context.Background()
	db, err := c.Database(ctx, "_system")
	if err != nil {
		// handle error
		log.Println(err)
		return nil, err
	}

	cursor, err := db.Query(ctx, AQLQuery, nil)
	if err != nil {
		log.Println("Invalid query") // handle error
		return nil, err
	}
	defer cursor.Close()

	//Loop through the resulting documents
	listContainer := ListContainer{}
	for {
		var doc map[string][]interface{}
		_, err := cursor.ReadDocument(ctx, &doc)
		if driver.IsNoMoreDocuments(err) {
			break
		} else if err != nil {
			// handle other errors
			return nil, err
		}

		parseResult(doc, &listContainer)
	}

	queryResult["nodes"] = listContainer.nodeList
	queryResult["edges"] = listContainer.edgeList

	//writeJSON(queryResult)
	//file, err := json.MarshalIndent(queryResult, "", " ")
	return &queryResult, nil
}

/* parseResult takes the result of the query and translates this to two lists: a nodelist and an edgelist, stored in a listcontainer

Parameters: doc is the document with the nodes and vertices that came back from the database,
listContainer is a struct containing the nodelist and edgelist that will be returned to the frontend

Return: Nothing because the result is stored in the listContainer
*/
func parseResult(doc map[string][]interface{}, listContainer *ListContainer) {
	vertices := doc["vertices"]
	edges := doc["edges"]

	fmt.Println(vertices)

	for _, vertex := range vertices {
		vertexDoc := vertex.(map[string]interface{})

		(*listContainer).nodeList = append((*listContainer).nodeList, parseNode(vertexDoc))
	}

	for _, edge := range edges {
		edgeDoc := edge.(map[string]interface{})

		(*listContainer).edgeList = append((*listContainer).edgeList, parseEdge(edgeDoc))
	}
}

/* parseEdge parses the data of an edge to an output-friendly format
Parameters: d is a single entry of an edge

Return: a document with almost the same structure as before, but the attributes are grouped
*/
func parseEdge(d map[string]interface{}) Document {
	doc := d //.(map[string]interface{})

	data := make(Document)
	data["_id"] = doc["_id"]
	delete(doc, "_id")
	data["_key"] = doc["_key"]
	delete(doc, "_key")
	data["_rev"] = doc["_rev"]
	delete(doc, "_rev")
	data["_from"] = doc["_from"]
	delete(doc, "_from")
	data["_to"] = doc["_to"]
	delete(doc, "_to")
	data["attributes"] = doc

	//delete(doc, "_key")
	//data.rev = fmt.Sprintf("%v", doc["_rev"])

	// delete(doc, "_rev")
	// data.attr = doc
	return data
}

// writeJSON writes a json file for testing purposes
func writeJSON(queryResult map[string][]Document) {
	file, _ := json.MarshalIndent(queryResult, "", " ")

	_ = ioutil.WriteFile("result.json", file, 0644)
}

/* parseEdge parses the data of a node to an output-friendly format
Parameters: d is a single entry of an node

Return: a document with almost the same structure as before, but the attributes are grouped
*/
func parseNode(d map[string]interface{}) Document {
	doc := d //.(map[string]interface{})

	data := make(Document)
	data["_id"] = doc["_id"]
	delete(doc, "_id")
	data["_key"] = doc["_key"]
	delete(doc, "_key")
	data["_rev"] = doc["_rev"]
	delete(doc, "_rev")
	data["attributes"] = doc

	//delete(doc, "_key")
	//data.rev = fmt.Sprintf("%v", doc["_rev"])

	// delete(doc, "_rev")
	// data.attr = doc
	return data
}
